package ru.ulstu.tsMethods.exponential;

import ru.ulstu.models.TimeSeries;
import ru.ulstu.models.exceptions.ModelingException;
import ru.ulstu.models.exceptions.TimeSeriesValidateException;
import ru.ulstu.tsMethods.TimeSeriesMethod;

import java.util.ArrayList;
import java.util.List;

import static ru.ulstu.tsMethods.exponential.ExponentialParamName.ALPHA;
import static ru.ulstu.tsMethods.exponential.ExponentialParamName.BETA;
import static ru.ulstu.tsMethods.exponential.ExponentialParamName.GAMMA;
import static ru.ulstu.tsMethods.exponential.ExponentialParamName.SEASON;

public class AddTrendAddSeason extends TimeSeriesMethod {
    private final ExponentialMethodParams exponentialMethodParams;
    private final List<Double> sComponent = new ArrayList<>();
    private final List<Double> tComponent = new ArrayList<>();
    private final List<Double> iComponent = new ArrayList<>();

    public AddTrendAddSeason(TimeSeries timeSeries, ExponentialMethodParams exponentialMethodParams) throws ModelingException {
        super(timeSeries);
        this.exponentialMethodParams = exponentialMethodParams;
    }

    @Override
    protected TimeSeries getModelOfValidTimeSeries() throws ModelingException {
        sComponent.clear();
        tComponent.clear();
        iComponent.clear();
        iComponent.add(1.0);
        sComponent.add(originalTimeSeries.getFirstValue().getValue());
        tComponent.add(0.0);
        TimeSeries model = new TimeSeries("Model of " + originalTimeSeries.getName());
        model.addValue(originalTimeSeries.getFirstValue());
        //выполняется проход модели по сглаживанию
        for (int t = 1; t < exponentialMethodParams.getValue(SEASON).intValue(); t++) {
            sComponent.add(exponentialMethodParams.getValue(ALPHA) * originalTimeSeries.getNumericValue(t)
                    + (1 - exponentialMethodParams.getValue(ALPHA))
                    * (sComponent.get(t - 1) + tComponent.get(t - 1)));
            tComponent.add(exponentialMethodParams.getValue(BETA)
                    * (sComponent.get(t) - sComponent.get(t - 1))
                    + (1 - exponentialMethodParams.getValue(BETA)) * tComponent.get(t - 1));
            iComponent.add(exponentialMethodParams.getValue(GAMMA) * originalTimeSeries.getNumericValue(t) / sComponent.get(sComponent.size() - 1)
                    + (1 - exponentialMethodParams.getValue(GAMMA)) * iComponent.get(0));
            model.addValue(originalTimeSeries.getValues().get(t), sComponent.get(sComponent.size() - 1));
        }
        for (int t = exponentialMethodParams.getValue(SEASON).intValue();
             t < originalTimeSeries.getValues().size(); t++) {
            sComponent.add(exponentialMethodParams.getValue(ALPHA) * originalTimeSeries.getNumericValue(t)
                    / iComponent.get(t - exponentialMethodParams.getValue(SEASON).intValue())
                    + (1 - exponentialMethodParams.getValue(ALPHA))
                    * (sComponent.get(t - 1) + tComponent.get(t - 1)));

            tComponent.add(exponentialMethodParams.getValue(BETA)
                    * (sComponent.get(t) - sComponent.get(t - 1))
                    + (1 - exponentialMethodParams.getValue(BETA)) * tComponent.get(t - 1));

            iComponent.add(exponentialMethodParams.getValue(GAMMA) * originalTimeSeries.getNumericValue(t) / sComponent.get(sComponent.size() - 1)
                    + (1 - exponentialMethodParams.getValue(GAMMA)) * iComponent.get(t - exponentialMethodParams.getValue(SEASON).intValue()));
            model.addValue(originalTimeSeries.getValues().get(t), sComponent.get(sComponent.size() - 1));
        }
        return model;
    }

    @Override
    protected void validateAdditionalParams() throws ModelingException {
        if (originalTimeSeries.getLength() < exponentialMethodParams.getValue(SEASON)) {
            throw new TimeSeriesValidateException("Период больше чем длина ряда");
        }
    }

    @Override
    protected TimeSeries makeForecast(TimeSeries forecast) throws ModelingException {
        for (int t = 1; t < forecast.getLength(); t++) {
            iComponent.add(exponentialMethodParams.getValue(GAMMA) * forecast.getNumericValue(t - 1) / sComponent.get(sComponent.size() - 1)
                    + (1 - exponentialMethodParams.getValue(GAMMA)) * iComponent.get(t + getModel().getLength() - exponentialMethodParams.getValue(SEASON).intValue()));

            forecast.getValues().get(t).setValue((sComponent.get(sComponent.size() - 1) + tComponent.get(tComponent.size() - 1) * t)
                    * iComponent.get(t + getModel().getLength() - exponentialMethodParams.getValue(SEASON).intValue()));
        }
        return forecast;
    }
}
